@@ -1,8 +1,8 @@
 # coding: utf-8
 
-VERSION = (0, 8, 1)
-CODENAME = "keeping track"
-BUILD_DT = (2021, 2, 22)
+VERSION = (0, 9, 11)
+CODENAME = "the strongest music server"
+BUILD_DT = (2021, 3, 23)
 
 S_VERSION = ".".join(map(str, VERSION))
 S_BUILD_DT = "{0:04d}-{1:02d}-{2:02d}".format(*BUILD_DT)
