@@ -41,11 +41,11 @@ class HttpCli(object):
         self.absolute_urls = False
         self.out_headers = {"Access-Control-Allow-Origin": "*"}
 
-    def log(self, msg):
-        self.log_func(self.log_src, msg)
+    def log(self, msg, c=0):
+        self.log_func(self.log_src, msg, c)
 
     def _check_nonfatal(self, ex):
-        return ex.code < 400 or ex.code == 404
+        return ex.code < 400 or ex.code in [404, 429]
 
     def _assert_safe_rem(self, rem):
         # sanity check to prevent any disasters
@@ -63,7 +63,7 @@ class HttpCli(object):
 
             if not headerlines[0]:
                 # seen after login with IE6.0.2900.5512.xpsp.080413-2111 (xp-sp3)
-                self.log("\033[1;31mBUG: trailing newline from previous request\033[0m")
+                self.log("BUG: trailing newline from previous request", c="1;31")
                 headerlines.pop(0)
 
             try:
@@ -74,7 +74,7 @@ class HttpCli(object):
         except Pebkac as ex:
             # self.log("pebkac at httpcli.run #1: " + repr(ex))
             self.keepalive = self._check_nonfatal(ex)
-            self.loud_reply(str(ex), status=ex.code)
+            self.loud_reply(unicode(ex), status=ex.code)
             return self.keepalive
 
         # time.sleep(0.4)
@@ -163,7 +163,7 @@ class HttpCli(object):
         response = ["HTTP/1.1 {} {}".format(status, HTTPCODE[status])]
 
         if length is not None:
-            response.append("Content-Length: " + str(length))
+            response.append("Content-Length: " + unicode(length))
 
         # close if unknown length, otherwise take client's preference
         response.append("Connection: " + ("Keep-Alive" if self.keepalive else "Close"))
@@ -222,6 +222,9 @@ class HttpCli(object):
             static_path = os.path.join(E.mod, "web/", self.vpath[5:])
             return self.tx_file(static_path)
 
+        if "tree" in self.uparam:
+            return self.tx_tree()
+
         # conditional redirect to single volumes
         if self.vpath == "" and not self.uparam:
             nread = len(self.rvol)
@@ -246,9 +249,6 @@ class HttpCli(object):
             self.vpath = None
             return self.tx_mounts()
 
-        if "tree" in self.uparam:
-            return self.tx_tree()
-
         return self.tx_browser()
 
     def handle_options(self):
@@ -323,8 +323,11 @@ class HttpCli(object):
         raise Pebkac(405, "don't know how to handle POST({})".format(ctype))
 
     def get_body_reader(self):
-        remains = int(self.headers.get("content-length", None))
-        if remains is None:
+        chunked = "chunked" in self.headers.get("transfer-encoding", "").lower()
+        remains = int(self.headers.get("content-length", -1))
+        if chunked:
+            return read_socket_chunked(self.sr), remains
+        elif remains == -1:
             self.keepalive = False
             return read_socket_unbounded(self.sr), remains
         else:
@@ -342,6 +345,10 @@ class HttpCli(object):
         with open(path, "wb", 512 * 1024) as f:
             post_sz, _, sha_b64 = hashcopy(self.conn, reader, f)
 
+        self.conn.hsrv.broker.put(
+            False, "up2k.hash_file", vfs.realpath, vfs.flags, rem, fn
+        )
+
         return post_sz, sha_b64, remains, path
 
     def handle_stash(self):
@@ -425,7 +432,7 @@ class HttpCli(object):
         body["ptop"] = vfs.realpath
         body["prel"] = rem
         body["addr"] = self.ip
-        body["flag"] = vfs.flags
+        body["vcfg"] = vfs.flags
 
         x = self.conn.hsrv.broker.put(True, "up2k.handle_json", body)
         response = x.get()
@@ -442,20 +449,42 @@ class HttpCli(object):
             vols.append([vfs.vpath, vfs.realpath, vfs.flags])
 
         idx = self.conn.get_u2idx()
+        t0 = time.time()
+        if idx.p_end:
+            penalty = 0.7
+            t_idle = t0 - idx.p_end
+            if idx.p_dur > 0.7 and t_idle < penalty:
+                m = "rate-limit ({:.1f} sec), cost {:.2f}, idle {:.2f}"
+                raise Pebkac(429, m.format(penalty, idx.p_dur, t_idle))
+
         if "srch" in body:
             # search by up2k hashlist
             vbody = copy.deepcopy(body)
             vbody["hash"] = len(vbody["hash"])
             self.log("qj: " + repr(vbody))
             hits = idx.fsearch(vols, body)
-            self.log("q#: " + repr(hits))
+            msg = repr(hits)
+            taglist = []
         else:
             # search by query params
             self.log("qj: " + repr(body))
-            hits = idx.search(vols, body)
-            self.log("q#: " + str(len(hits)))
+            hits, taglist = idx.search(vols, body)
+            msg = len(hits)
 
-        r = json.dumps(hits).encode("utf-8")
+        idx.p_end = time.time()
+        idx.p_dur = idx.p_end - t0
+        self.log("q#: {} ({:.2f}s)".format(msg, idx.p_dur))
+
+        order = []
+        cfg = self.args.mte.split(",")
+        for t in cfg:
+            if t in taglist:
+                order.append(t)
+        for t in taglist:
+            if t not in order:
+                order.append(t)
+
+        r = json.dumps({"hits": hits, "tag_order": order}).encode("utf-8")
         self.reply(r, mime="application/json")
         return True
 
@@ -503,7 +532,7 @@ class HttpCli(object):
             if len(cstart) > 1 and path != os.devnull:
                 self.log(
                     "clone {} to {}".format(
-                        cstart[0], " & ".join(str(x) for x in cstart[1:])
+                        cstart[0], " & ".join(unicode(x) for x in cstart[1:])
                     )
                 )
                 ofs = 0
@@ -661,6 +690,9 @@ class HttpCli(object):
                             raise Pebkac(400, "empty files in post")
 
                         files.append([sz, sha512_hex])
+                        self.conn.hsrv.broker.put(
+                            False, "up2k.hash_file", vfs.realpath, vfs.flags, rem, fname
+                        )
                         self.conn.nbyte += sz
 
                 except Pebkac:
@@ -676,7 +708,7 @@ class HttpCli(object):
                     raise
 
         except Pebkac as ex:
-            errmsg = str(ex)
+            errmsg = unicode(ex)
 
         td = max(0.1, time.time() - t0)
         sz_total = sum(x[0] for x in files)
@@ -985,7 +1017,7 @@ class HttpCli(object):
             mime=guess_mime(req_path)[0] or "application/octet-stream",
         )
 
-        logmsg += str(status) + logtail
+        logmsg += unicode(status) + logtail
 
         if self.mode == "HEAD" or not do_send:
             self.log(logmsg)
@@ -999,7 +1031,7 @@ class HttpCli(object):
                 remains = sendfile_py(lower, upper, f, self.s)
 
         if remains > 0:
-            logmsg += " \033[31m" + str(upper - remains) + "\033[0m"
+            logmsg += " \033[31m" + unicode(upper - remains) + "\033[0m"
 
         spd = self._spd((upper - lower) - remains)
         self.log("{},  {}".format(logmsg, spd))
@@ -1046,7 +1078,7 @@ class HttpCli(object):
         sz_html = len(template.render(**targs).encode("utf-8"))
         self.send_headers(sz_html + sz_md, status)
 
-        logmsg += str(status)
+        logmsg += unicode(status)
         if self.mode == "HEAD" or not do_send:
             self.log(logmsg)
             return True
@@ -1060,7 +1092,7 @@ class HttpCli(object):
             self.log(logmsg + " \033[31md/c\033[0m")
             return False
 
-        self.log(logmsg + " " + str(len(html)))
+        self.log(logmsg + " " + unicode(len(html)))
         return True
 
     def tx_mounts(self):
@@ -1094,11 +1126,12 @@ class HttpCli(object):
         excl = None
         if target:
             excl, target = (target.split("/", 1) + [""])[:2]
-            ret["k" + excl] = self.gen_tree("/".join([top, excl]).strip("/"), target)
+            sub = self.gen_tree("/".join([top, excl]).strip("/"), target)
+            ret["k" + quotep(excl)] = sub
 
         try:
             vn, rem = self.auth.vfs.get(top, self.uname, True, False)
-            fsroot, vfs_ls, vfs_virt = vn.ls(rem, self.uname)
+            fsroot, vfs_ls, vfs_virt = vn.ls(rem, self.uname, not self.args.no_scandir)
         except:
             vfs_ls = []
             vfs_virt = {}
@@ -1109,13 +1142,13 @@ class HttpCli(object):
 
         dirs = []
 
+        vfs_ls = [x[0] for x in vfs_ls if stat.S_ISDIR(x[1].st_mode)]
+
         if not self.args.ed or "dots" not in self.uparam:
             vfs_ls = exclude_dotfiles(vfs_ls)
 
         for fn in [x for x in vfs_ls if x != excl]:
-            abspath = os.path.join(fsroot, fn)
-            if os.path.isdir(abspath):
-                dirs.append(fn)
+            dirs.append(quotep(fn))
 
         for x in vfs_virt.keys():
             if x != excl:
@@ -1154,7 +1187,9 @@ class HttpCli(object):
 
             return self.tx_file(abspath)
 
-        fsroot, vfs_ls, vfs_virt = vn.ls(rem, self.uname)
+        fsroot, vfs_ls, vfs_virt = vn.ls(rem, self.uname, not self.args.no_scandir)
+        stats = {k: v for k, v in vfs_ls}
+        vfs_ls = [x[0] for x in vfs_ls]
         vfs_ls.extend(vfs_virt.keys())
 
         # check for old versions of files,
@@ -1183,6 +1218,11 @@ class HttpCli(object):
 
         is_ls = "ls" in self.uparam
 
+        icur = None
+        if "e2t" in vn.flags:
+            idx = self.conn.get_u2idx()
+            icur = idx.get_cur(vn.realpath)
+
         dirs = []
         files = []
         for fn in vfs_ls:
@@ -1200,7 +1240,7 @@ class HttpCli(object):
                 fspath = fsroot + "/" + fn
 
             try:
-                inf = os.stat(fsenc(fspath))
+                inf = stats.get(fn) or os.stat(fsenc(fspath))
             except:
                 self.log("broken symlink: {}".format(repr(fspath)))
                 continue
@@ -1232,18 +1272,49 @@ class HttpCli(object):
                 "sz": sz,
                 "ext": ext,
                 "dt": dt,
-                "ts": inf.st_mtime,
+                "ts": int(inf.st_mtime),
             }
             if is_dir:
                 dirs.append(item)
             else:
                 files.append(item)
+                item["rd"] = rem
+
+        taglist = {}
+        for f in files:
+            fn = f["name"]
+            rd = f["rd"]
+            del f["rd"]
+            if icur:
+                q = "select w from up where rd = ? and fn = ?"
+                try:
+                    r = icur.execute(q, (rd, fn)).fetchone()
+                except:
+                    args = s3enc(idx.mem_cur, rd, fn)
+                    r = icur.execute(q, args).fetchone()
+
+                tags = {}
+                f["tags"] = tags
+                
+                if not r:
+                    continue
+
+                w = r[0][:16]
+                q = "select k, v from mt where w = ? and k != 'x'"
+                for k, v in icur.execute(q, (w,)):
+                    taglist[k] = True
+                    tags[k] = v
+
+        if icur:
+            taglist = [k for k in vn.flags["mte"].split(",") if k in taglist]
+            for f in dirs:
+                f["tags"] = {}
 
         srv_info = []
 
         try:
             if not self.args.nih:
-                srv_info.append(str(socket.gethostname()).split(".")[0])
+                srv_info.append(unicode(socket.gethostname()).split(".")[0])
         except:
             self.log("#wow #whoa")
             pass
@@ -1275,20 +1346,27 @@ class HttpCli(object):
         if self.writable:
             perms.append("write")
 
-        if is_ls:
-            [x.pop(k) for k in ["name", "dt"] for y in [dirs, files] for x in y]
-            ret = {"dirs": dirs, "files": files, "srvinf": srv_info, "perms": perms}
-            ret = json.dumps(ret)
-            self.reply(ret.encode("utf-8", "replace"), mime="application/json")
-            return True
-
-        logues = [None, None]
+        logues = ["", ""]
         for n, fn in enumerate([".prologue.html", ".epilogue.html"]):
             fn = os.path.join(abspath, fn)
             if os.path.exists(fsenc(fn)):
                 with open(fsenc(fn), "rb") as f:
                     logues[n] = f.read().decode("utf-8")
 
+        if is_ls:
+            [x.pop(k) for k in ["name", "dt"] for y in [dirs, files] for x in y]
+            ret = {
+                "dirs": dirs,
+                "files": files,
+                "srvinf": srv_info,
+                "perms": perms,
+                "logues": logues,
+                "taglist": taglist,
+            }
+            ret = json.dumps(ret)
+            self.reply(ret.encode("utf-8", "replace"), mime="application/json")
+            return True
+
         ts = ""
         # ts = "?{}".format(time.time())
 
@@ -1300,9 +1378,11 @@ class HttpCli(object):
             files=dirs,
             ts=ts,
             perms=json.dumps(perms),
-            have_up2k_idx=self.args.e2d,
-            prologue=logues[0],
-            epilogue=logues[1],
+            taglist=taglist,
+            tag_order=json.dumps(vn.flags["mte"].split(",")),
+            have_up2k_idx=("e2d" in vn.flags),
+            have_tags_idx=("e2t" in vn.flags),
+            logues=logues,
             title=html_escape(self.vpath),
             srv_info=srv_info,
         )
