@@ -46,9 +46,9 @@ function up2k_flagbus() {
     var dbg = function (who, msg) {
         console.log('flagbus(' + flag.id + '): [' + who + '] ' + msg);
     };
-    flag.ch.onmessage = function (ev) {
-        var who = ev.data[0],
-            what = ev.data[1];
+    flag.ch.onmessage = function (e) {
+        var who = e.data[0],
+            what = e.data[1];
 
         if (who == flag.id) {
             dbg(who, 'hi me (??)');
@@ -83,7 +83,7 @@ function up2k_flagbus() {
             flag.ch.postMessage([flag.id, "hey"]);
         }
         else {
-            dbg('?', ev.data);
+            dbg('?', e.data);
         }
     };
     var tx = function (now, msg) {
@@ -194,7 +194,7 @@ function up2k_init(have_crypto) {
 
     // handle user intent to use the basic uploader instead
     ebi('u2nope').onclick = function (e) {
-        e.preventDefault();
+        ev(e);
         setmsg();
         goto('bup');
     };
@@ -209,42 +209,7 @@ function up2k_init(have_crypto) {
         };
     }
 
-    function cfg_get(name) {
-        var val = localStorage.getItem(name);
-        if (val === null)
-            return parseInt(ebi(name).value);
-
-        ebi(name).value = val;
-        return val;
-    }
-
-    function bcfg_get(name, defval) {
-        var o = ebi(name);
-        if (!o)
-            return defval;
-
-        var val = localStorage.getItem(name);
-        if (val === null)
-            val = defval;
-        else
-            val = (val == '1');
-
-        o.checked = val;
-        return val;
-    }
-
-    function bcfg_set(name, val) {
-        localStorage.setItem(
-            name, val ? '1' : '0');
-
-        var o = ebi(name);
-        if (o)
-            o.checked = val;
-
-        return val;
-    }
-
-    var parallel_uploads = cfg_get('nthread');
+    var parallel_uploads = icfg_get('nthread');
     var multitask = bcfg_get('multitask', true);
     var ask_up = bcfg_get('ask_up', true);
     var flag_en = bcfg_get('flag_en', false);
@@ -282,36 +247,36 @@ function up2k_init(have_crypto) {
 
     var flag = false;
     apply_flag_cfg();
-    apply_fsearch_cfg();
+    set_fsearch();
 
     function nav() {
         ebi('file' + fdom_ctr).click();
     }
     ebi('u2btn').addEventListener('click', nav, false);
 
-    function ondrag(ev) {
-        ev.stopPropagation();
-        ev.preventDefault();
-        ev.dataTransfer.dropEffect = 'copy';
-        ev.dataTransfer.effectAllowed = 'copy';
+    function ondrag(e) {
+        e.stopPropagation();
+        e.preventDefault();
+        e.dataTransfer.dropEffect = 'copy';
+        e.dataTransfer.effectAllowed = 'copy';
     }
     ebi('u2btn').addEventListener('dragover', ondrag, false);
     ebi('u2btn').addEventListener('dragenter', ondrag, false);
 
-    function gotfile(ev) {
-        ev.stopPropagation();
-        ev.preventDefault();
+    function gotfile(e) {
+        e.stopPropagation();
+        e.preventDefault();
 
         var files;
         var is_itemlist = false;
-        if (ev.dataTransfer) {
-            if (ev.dataTransfer.items) {
-                files = ev.dataTransfer.items; // DataTransferItemList
+        if (e.dataTransfer) {
+            if (e.dataTransfer.items) {
+                files = e.dataTransfer.items; // DataTransferItemList
                 is_itemlist = true;
             }
-            else files = ev.dataTransfer.files; // FileList
+            else files = e.dataTransfer.files; // FileList
         }
-        else files = ev.target.files;
+        else files = e.target.files;
 
         if (files.length == 0)
             return alert('no files selected??');
@@ -367,7 +332,7 @@ function up2k_init(have_crypto) {
                 "name": fobj.name,
                 "size": fobj.size,
                 "lmod": lmod / 1000,
-                "purl": get_vpath(),
+                "purl": get_evpath(),
                 "done": false,
                 "hash": []
             };
@@ -690,8 +655,8 @@ function up2k_init(have_crypto) {
             prog(t.n, nchunk, col_hashing);
         };
 
-        var segm_load = function (ev) {
-            cache_buf = ev.target.result;
+        var segm_load = function (e) {
+            cache_buf = e.target.result;
             cache_ofs = 0;
             hash_calc();
         };
@@ -765,20 +730,20 @@ function up2k_init(have_crypto) {
         st.busy.handshake.push(t);
 
         var xhr = new XMLHttpRequest();
-        xhr.onload = function (ev) {
+        xhr.onload = function (e) {
             if (xhr.status == 200) {
                 var response = JSON.parse(xhr.responseText);
 
                 if (!response.name) {
                     var msg = '';
                     var smsg = '';
-                    if (!response || !response.length) {
+                    if (!response || !response.hits || !response.hits.length) {
                         msg = 'not found on server';
                         smsg = '404';
                     }
                     else {
                         smsg = 'found';
-                        var hit = response[0],
+                        var hit = response.hits[0],
                             msg = linksplit(hit.rp).join(''),
                             tr = unix2iso(hit.ts),
                             tu = unix2iso(t.lmod),
@@ -916,7 +881,7 @@ function up2k_init(have_crypto) {
             alert('y o u   b r o k e    i t\n\n(was that a folder? just files please)');
         };
 
-        reader.onload = function (ev) {
+        reader.onload = function (e) {
             var xhr = new XMLHttpRequest();
             xhr.upload.onprogress = function (xev) {
                 var perc = xev.loaded / (cdr - car) * 100;
@@ -950,7 +915,7 @@ function up2k_init(have_crypto) {
             xhr.setRequestHeader('Content-Type', 'application/octet-stream');
             xhr.overrideMimeType('Content-Type', 'application/octet-stream');
             xhr.responseType = 'text';
-            xhr.send(ev.target.result);
+            xhr.send(e.target.result);
         };
 
         reader.readAsArrayBuffer(bobslice.call(t.fobj, car, cdr));
@@ -979,7 +944,7 @@ function up2k_init(have_crypto) {
     ///   config ui
     //
 
-    function onresize(ev) {
+    function onresize(e) {
         var bar = ebi('ops'),
             wpx = innerWidth,
             fpx = parseInt(getComputedStyle(bar)['font-size']),
@@ -994,17 +959,17 @@ function up2k_init(have_crypto) {
             ebi('u2conf').setAttribute('class', wide ? 'has_btn' : '');
         }
     }
-    window.onresize = onresize;
+    window.addEventListener('resize', onresize);
     onresize();
 
-    function desc_show(ev) {
+    function desc_show(e) {
         var msg = this.getAttribute('alt');
         msg = msg.replace(/\$N/g, "<br />");
         var cdesc = ebi('u2cdesc');
         cdesc.innerHTML = msg;
         cdesc.setAttribute('class', 'show');
     }
-    function desc_hide(ev) {
+    function desc_hide(e) {
         ebi('u2cdesc').setAttribute('class', '');
     }
     var o = document.querySelectorAll('#u2conf *[alt]');
@@ -1033,7 +998,7 @@ function up2k_init(have_crypto) {
                 return;
 
             parallel_uploads = v;
-            localStorage.setItem('nthread', v);
+            swrite('nthread', v);
             obj.style.background = '#444';
             return;
         }
@@ -1061,12 +1026,31 @@ function up2k_init(have_crypto) {
     }
 
     function tgl_fsearch() {
-        fsearch = !fsearch;
-        bcfg_set('fsearch', fsearch);
-        apply_fsearch_cfg();
+        set_fsearch(!fsearch);
     }
 
-    function apply_fsearch_cfg() {
+    function set_fsearch(new_state) {
+        var perms = document.body.getAttribute('perms');
+        var read_only = false;
+
+        if (!ebi('fsearch')) {
+            new_state = false;
+        }
+        else if (perms && perms.indexOf('write') === -1) {
+            new_state = true;
+            read_only = true;
+        }
+
+        if (new_state !== undefined) {
+            fsearch = new_state;
+            bcfg_set('fsearch', fsearch);
+        }
+
+        try {
+            document.querySelector('label[for="fsearch"]').style.opacity = read_only ? '0' : '1';
+        }
+        catch (ex) { }
+
         try {
             var fun = fsearch ? 'add' : 'remove';
             ebi('op_up2k').classList[fun]('srch');
@@ -1078,11 +1062,6 @@ function up2k_init(have_crypto) {
         catch (ex) { }
     }
 
-    function set_fsearch() {
-        if (!fsearch)
-            tgl_fsearch();
-    }
-
     function tgl_flag_en() {
         flag_en = !flag_en;
         bcfg_set('flag_en', flag_en);
@@ -1105,17 +1084,17 @@ function up2k_init(have_crypto) {
         }
     }
 
-    function nop(ev) {
-        ev.preventDefault();
+    function nop(e) {
+        ev(e);
         this.click();
     }
 
-    ebi('nthread_add').onclick = function (ev) {
-        ev.preventDefault();
+    ebi('nthread_add').onclick = function (e) {
+        ev(e);
         bumpthread(1);
     };
-    ebi('nthread_sub').onclick = function (ev) {
-        ev.preventDefault();
+    ebi('nthread_sub').onclick = function (e) {
+        ev(e);
         bumpthread(-1);
     };
 
@@ -1131,12 +1110,8 @@ function up2k_init(have_crypto) {
     for (var a = nodes.length - 1; a >= 0; a--)
         nodes[a].addEventListener('touchend', nop, false);
 
-    var perms = document.body.getAttribute('perms');
-    if (perms && perms.indexOf('write') === -1)
-        set_fsearch();
-
+    set_fsearch();
     bumpthread({ "target": 1 })
-
     return { "init_deps": init_deps, "set_fsearch": set_fsearch }
 }
 
